@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <config.h> 
 #include <kdeversion.h>
 #include <klocale.h>
 #include <kglobal.h>
@@ -36,6 +37,10 @@
 #include "kbconfig.h"
 #include "xkeyboard.h"
 
+#ifdef HAVE_X11_EXTENSIONS_XKBRULES_H
+  #include <X11/extensions/XKBrules.h>
+#endif
+
 #define TOGGLE_MODE_ENTRY "toggle_mode"
 #define DEFAULT_GROUP_ENTRY "default_group"
 //#define PERWINDOW_GROUP_ENTRY "perwindow_group"
@@ -123,32 +128,37 @@ void KBConfig::drawIcons(IconStyle icon_style, QValueVector<QPixmap> *icons,
   QStringList *iconpaths)
 {
   QString path, countryCode, langCode;
+  QStringList layouts;
+  
   KGlobal::dirs()->addResourceDir("appdata", ".");
   KConfig map("group_names", true, true, "appdata");
   KConfig *config = kapp->config();
   config->setGroup(ICONS_SECTION);
+  getXkbLayouts(layouts);
 
-	icons->clear();
-	icons->resize(m_groups.count());
-	iconpaths->clear();
-	for (unsigned int i = 0; i < m_groups.count(); i++) {
-	  map.setGroup("Mapping");
+  icons->clear();
+  icons->resize(m_groups.count());
+  iconpaths->clear();
+  for (unsigned int i = 0; i < m_groups.count(); i++) {
+    map.setGroup("Mapping");
     countryCode = map.readEntry(m_groups[i]->getName());
-	  map.setGroup("Languages");
+    // if the country code can't be guessed from group name, try XKB layout name
+    if (countryCode.isEmpty() && i < layouts.count()) countryCode = layouts[i];
+    map.setGroup("Languages");
     langCode = map.readEntry(m_groups[i]->getName());
     if (langCode.isEmpty()) langCode = countryCode;
-		QPixmap &pix = (*icons)[i];
-		pix.resize(0, 0);
-		path = QString::null;
-  	if (icon_style == ICON_FLAG /*&& !countryCode.isEmpty()*/)
-	    drawFlagPixmap(pix, path, countryCode, i, config);
-	  else if (icon_style == ICON_CODE_AND_FLAG /*&& !countryCode.isEmpty()*/)
-	 	  drawCodeAndFlagPixmap(pix, path, countryCode, langCode, i, config);
-  	if (pix.isNull() && !langCode.isEmpty())
-      drawCodePixmap(pix, langCode);
+    QPixmap &pix = (*icons)[i];
+    pix.resize(0, 0);
+    path = QString::null;
+    if (icon_style == ICON_FLAG /*&& !countryCode.isEmpty()*/)
+      drawFlagPixmap(pix, path, countryCode, i, config);
+    else if (icon_style == ICON_CODE_AND_FLAG /*&& !countryCode.isEmpty()*/)
+      drawCodeAndFlagPixmap(pix, path, countryCode, langCode, i, config);
+    if (pix.isNull() && !langCode.isEmpty())
+    drawCodePixmap(pix, langCode);
     if (pix.isNull()) drawDefaultPixmap(pix, i);
     iconpaths->append(path);
-	}	
+  }
 }
 
 /** No descriptions */
@@ -180,29 +190,28 @@ void KBConfig::drawIcons(IconStyle icon_style, QValueVector<QPixmap> *icons,
 bool KBConfig::getGroupImage(QImage &img, QString &path, const QString &code,
   int group, KConfig *config)
 {
-	bool ret = true;
-	bool need_to_scale = false;
+  bool ret = true;
+  bool need_to_scale = false;
 	
-	path = config->readEntry(m_groups[group]->getName()/*entryForGroup(group)*/);
-	if (path.isEmpty() || !img.load(path)) {
-	  if (code.isEmpty()) ret = false;
-	  else {	
+  path = config->readEntry(m_groups[group]->getName()/*entryForGroup(group)*/);
+  if (path.isEmpty() || !img.load(path)) {
+    if (code.isEmpty()) ret = false;
+    else {
       path = locate("locale", QString("l10n/%1/flag.png").arg(code));
       if (!path.isEmpty()) ret = img.load(path);
       else {
-  			// I am told in Red Hat 9 standard KDE flag pixmaps are missing.
-  			// Workaround: we have to simulate them by rescaling GKB's pixmaps
+        // I am told in Red Hat 9 standard KDE flag pixmaps are missing.
+        // Workaround: we have to simulate them by rescaling GKB's pixmaps
         path = QString("/usr/share/pixmaps/gkb/%1.png").arg(code);
-  			img.load(path);
-  			need_to_scale = true;
-  			if (img.isNull()) ret = false;
-  		}
+        ret = img.load(path) && !img.isNull();
+        need_to_scale = true;
+      }
     } 
   }
   if (ret) {
-		// if need_to_scale is not already set, set it if image is too wide
-		if (!need_to_scale) need_to_scale = img.width() > 24;
-		if (need_to_scale) img = img.smoothScale(FLAG_ICON_WIDTH, FLAG_ICON_HEIGHT);
+     // if need_to_scale is not already set, set it if image is too wide
+     if (!need_to_scale) need_to_scale = img.width() > 24;
+     if (need_to_scale) img = img.smoothScale(FLAG_ICON_WIDTH, FLAG_ICON_HEIGHT);
   }
   return ret;
 }
@@ -210,8 +219,8 @@ bool KBConfig::getGroupImage(QImage &img, QString &path, const QString &code,
 void KBConfig::drawFlagPixmap(QPixmap &pix, QString &path, const QString &code,
   int group, KConfig *config)
 {
-	QImage img;
-	if (getGroupImage(img, path, code, group, config)) 
+  QImage img;
+  if (getGroupImage(img, path, code, group, config)) 
     pix.convertFromImage(img);
 }
 
@@ -303,3 +312,29 @@ void KBConfig::checkKeysEnabled()
 /*void KBConfig::notifyChanged(){
   emit changed();
 }*/
+
+
+/*!
+    \fn KBConfig::getXkbLayouts(const QStringList &layouts)
+ */
+void KBConfig::getXkbLayouts(QStringList &layouts)
+{
+#if HAVE_X11_EXTENSIONS_XKBRULES_H && HAVE_LIBXKBFILE
+  XkbRF_VarDefsRec vardefs;
+  int i;
+
+  usleep(10000);
+  if (XkbRF_GetNamesProp(qt_xdisplay(), NULL, &vardefs)) {
+    layouts = QStringList::split(',', vardefs.layout, true);
+    for (QStringList::Iterator it = layouts.begin(); it != layouts.end(); ++it) {
+      i = 0;
+      while ((*it)[i] >= 'a' && (*it)[i] <= 'z') i++;
+      *it = (*it).left(i);
+    }
+    if (vardefs.layout) XFree(vardefs.layout);
+    if (vardefs.model) XFree(vardefs.model);
+    if (vardefs.variant) XFree(vardefs.variant);
+    if (vardefs.options) XFree(vardefs.options);
+  }
+#endif
+}
