@@ -24,17 +24,3 @@ KBGroup::KBGroup(const QString& name){
 
 KBGroup::~KBGroup(){
 }
-
-QPixmap& KBGroup::getPixmap() {
-  return m_pixmap;
-}
-
-/** No descriptions */
-QString KBGroup::getName(){
-  return m_name;
-}
-
-/** No descriptions */
-void KBGroup::setPixmap(QPixmap& pixmap){
-  m_pixmap = pixmap;
-}
