@@ -51,11 +51,11 @@ typedef QMap<WId, KBWinInfo> KBWindowMap;*/
   *@author Leonid Zeitlin
   */
 
-class KBSwitchApp : public /*KApplication*/ KUniqueApplication {
+class KBSwitchApp : public KUniqueApplication {
   Q_OBJECT
 public: 
-	KBSwitchApp();
-	~KBSwitchApp();
+  KBSwitchApp();
+  ~KBSwitchApp();
 protected: // Protected methods
   /** No descriptions */
   bool x11EventFilter(XEvent *e);
